#include <stdio.h>
#include <fcntl.h>
#include <termios.h>
#include <sys/time.h>
#include <sys/types.h>
#include <unistd.h>
#include <libgen.h>
#include <string.h>

#define ARRAYSIZE(x) (sizeof(x) / sizeof(x[0]))

int port_fd;		/* FD for serial port */

struct {
   char *baud_str;
   long bits_per_sec;
   long baud_val;
} baud_lookup[] =
{
   { "50",50,B50},
   { "75",75,B75},
   { "110",110,B110},
   { "134",134,B134},
   { "150",150,B150},
   { "200",200,B200},
   { "300",300,B300},
   { "600",600,B600},
   { "1200",1200,B1200},
   { "1800",1800,B1800},
   { "2400",2400,B2400},
   { "4800",4800,B4800},
   { "9600",9600,B9600},
   { "19200",19200,B19200}
#ifdef B38400
   ,{ "38400",38400,B38400}
#endif
#ifdef B57600
   ,{ "57600",57600,B57600}
#endif
#ifdef B115200
   ,{ "115200",115200,B115200}
#endif
#ifdef B230400
   ,{ "230400",230400,B230400}
#endif
#ifdef B460800
   ,{ "460800",460800,B460800}
#endif
};


/* convert integer baud rate into B#### definition */
int conv_baud(int baud_rate)
{
   int cntr;
   int baud;

   baud = B0;
   for (cntr = 0; cntr < ARRAYSIZE(baud_lookup) && baud == B0; cntr++) {
      if (baud_lookup[cntr].bits_per_sec == baud_rate) {
         baud = baud_lookup[cntr].baud_val;
      }
   }
   if (baud == B0) {
      printf("Unknown baud rate %d\n",baud_rate);
      exit(1);
   }

   return baud;
}

/* Setup serial port */
void AsyncSet(int wBaudRate, int wByteSize, int wStopBits, int wParity)
{
   struct termios tios;              /* Serial port TERMIO structure */
   int baud;

      /* Set to 8 bit selected baud no parity with no special processing */
   if (tcgetattr(port_fd,&tios) < 0) {
      perror("init_comm: tcgetattr failed");
      byebye(7);
   }
   baud = conv_baud(wBaudRate);
   tios.c_cflag = CREAD | HUPCL | CLOCAL | baud;
   if (wByteSize == 8)
      tios.c_cflag |= CS8;
   if (wStopBits != 0)
      tios.c_cflag |= CSTOPB;
   tios.c_iflag = 0;
   tios.c_lflag = 0;
   tios.c_oflag = 0;
   tios.c_cc[VMIN] = 0;
   tios.c_cc[VTIME] = 1;

   if (cfsetispeed(&tios,baud) != 0)
      printf("AsyncSet: set ispeed failed\n");
   if (cfsetospeed(&tios,baud) != 0)
      printf("AsyncSet: set ospeed failed\n");

   if (tcsetattr(port_fd,TCSANOW,&tios) < 0) {
      perror("AsyncSet: tcsetattr failed");
      byebye(7);
   }
   tcflush(port_fd,TCIOFLUSH);
}

/* Open serial port */
int AsyncInit(char *port) {
   port_fd = open(port,O_RDWR,0);
   if (port_fd < 0) {
      fprintf (stderr,"AsyncSet: Open failed on port '%s': ",port);
      perror("");
      byebye(7);
   }
   return 0;
}

/* Close serial port */
void AsyncStop(void) {
   close(port_fd);
}

/* Send bytes to serial port */
void AsyncOut(unsigned char *msg, int count) {
   int rc;

   if ((rc = write(port_fd, msg, count)) != count) {
      fprintf(stderr, "AsyncOut: Write failed %d\n",rc);
      byebye(7);
   }
}

/* Read a byte from serial port */
int AsyncIn(void)
{
   unsigned char byte;
   int rc;

   rc = read(port_fd, &byte, 1);
   if (rc < 0) {
      perror("AsyncIn: read failed");
      byebye(7);
   }
   if (rc) 
      return byte;
   else
      return 1000;
}

/* Return 1 if serial character available to read otherwise sleep 10 ms
   and return 0 */
int AsyncInStat(void)
{
   fd_set rmaskfd,wmaskfd,emaskfd;   /* Select masks */
   struct timeval timeout;
   int rc;

   FD_ZERO(&rmaskfd);
   FD_ZERO(&wmaskfd);
   FD_ZERO(&emaskfd);

   FD_SET(port_fd ,&rmaskfd);

   timeout.tv_sec = 0;
   timeout.tv_usec = 10000;

   rc = select(port_fd+1,&rmaskfd,&wmaskfd,&emaskfd,&timeout);
   if (rc < 0) {
      perror("AsyncInState: select failed");
      byebye(7);
   }
   if (rc > 0 && FD_ISSET(port_fd, &rmaskfd)) 
      return 1;
   else
      return 0;

}

/* Return key hit, not implemented under unix use control C to abort */
int kbraw(void)
{
#if 0
   fd_set rmaskfd,wmaskfd,emaskfd;   /* Select masks */
   struct timeval timeout;
   int rc;

   FD_ZERO(&rmaskfd);
   FD_ZERO(&wmaskfd);
   FD_ZERO(&emaskfd);

   FD_SET(STDIN_FILENO ,&rmaskfd);

   timeout.tv_sec = 0;
   timeout.tv_usec = 0;

   rc = select(port_fd+1,&rmaskfd,&wmaskfd,&emaskfd,&timeout);
   if (rc < 0) {
      perror("kbraw: select failed");
      byebye(7);
   }
   if (rc > 0 && FD_ISSET(port_fd, &rmaskfd)) 
      return getc(stdin);
   else 
#endif
      return 0;
}

/* Return 1 if serial port can't receive another character
  Not used, write will block if serial port buffer fills
*/
int AsyncOutStat(void){return 0;}

/* Convert string to lower case */
char *strlwr(char *str) {
   char *orig = str;
   while (*str != 0) {
      *str = tolower(*str);
      str++;
   }
   return orig;
}

/* Convert string to upper case */
char *strupr(char *str)
{
   char *orig = str;
   while (*str != 0) {
      *str = toupper(*str);
      *str++;
   }
   return orig;
}

/* Stuff for microsoft windows version, not used under unix */
void consoff(void) {
}

/* Set window title, not implemented under unix */
void SetConsoleTitle(void) {
}

/* Search specified enviorment varialbe for file.  Not implemented for unix */
_searchenv(char *searchfile,char *envvar,char *pathbuffer ) {
  *pathbuffer = 0; 
}

/* Split filename into separate components */
void fnsplit(char *fname_in,char *drive,char *dir,char *file,char *ext) {
   char *loc;

   *drive = 0;
   strcpy(dir,dirname(fname_in));
   strcpy(file,basename(fname_in));
   loc = strrchr(file,'.');
   if (loc != NULL) {
      strcpy(ext, loc+1);
      *loc = 0;
   }
}
